#!/usr/bin/perl
#
# fixcl - cleanup files leftover or made obsolete by fix
# Copyright (c) 2023 SAS Institute Inc.
#

use File::Copy;
use File::Path;
use strict;

# globals
my ($home, $di_dir);
my $debug;

# find where this script is stored
sub get_mydir {
    my ($me, $dir);

    if ($0 =~ m:^/:) {
        $me = $0;
    } else {
        chomp($dir = `pwd`);
        $me = "$dir/$0";
    }
    ($dir = $me) =~ s:(.*)/.*:$1:;
    $dir =~ s:/.$::;
    $dir =~ s:/+$::;
    return $dir;
}

sub dbg_print {
    return 1 if !$debug;
    my $msg = shift;
    print STDERR "[debug] $msg";
    if ($msg !~ /\n$/) {
        print STDERR "\n";
    }
}

# if $DFEXEC_HOME is not set, assume it is the parent of this script's 
# directory since this script should be installed in $DFEXEC_HOME/bin 
$home = $ENV{'DFEXEC_HOME'};
if (!$home) {
    $home = get_mydir();
    $home =~ s:/.$::;
    $home =~ s:/+$::;
    $home =~ s:/bin$::;
    $ENV{'DFEXEC_HOME'} = $home;
}

# file locations
if (-d "$home/data/install") {
    $di_dir = "$home/data/install";
} else {
    $di_dir = $home;
}

# command-line options
my $usage = "$0 <id>\n";
my $fix_id;
my $arg = shift;
if ($arg) {
    if ($arg eq '-d') {
        $debug = 1;
        $fix_id = shift;
    } else {
        $fix_id = $arg;
    }
} else {
    print "usage: $usage\n";
    exit(1);
}

# some debug messages
dbg_print("data/install = $di_dir");

# start message
print "cleaning files for patch fix$fix_id...\n";

# open remove list for patch fixXXXXX and remove each file that exists
my $file_count = 0;
my $rm_count = 0;
my $remove_list = "$di_dir/remove.fix$fix_id.meta";
dbg_print("reading $remove_list");
if (-f $remove_list) {
    open(RL, "< $remove_list") || die "open: $!";
    while (<RL>) {
        next if /^#/;
        s/^\s+//;
        s/\s+$//;
        my $file = "$home/$_";
        dbg_print("checking for $file");
        if (-f $file) {
            $file_count++;
            print "removing $file\n";
            if (unlink($file)) {
                $rm_count++;
            } else {
                print "error: $!\n";
            }
        }
    }
    close(RL);
}

# summary of actions
if ($file_count) {
    print "removed $rm_count files\n";
} else {
    print "no files to remove.\n";
}

